package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.Calendar;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.FechamentoParamgeralDAO;
import com.rjconsultores.ventaboletos.entidad.FechamentoParamgeral;
import com.rjconsultores.ventaboletos.entidad.FechamentoParamptovta;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Repository("fechamentoParamgeralDAO")
public class FechamentoParamgeralHibernateDAO extends GenericHibernateDAO<FechamentoParamgeral, Long> implements FechamentoParamgeralDAO {

	
	@Autowired
	public FechamentoParamgeralHibernateDAO(
			@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}
	
	@Override
	public List<FechamentoParamgeral> buscaParametrosPorEmpresas(List<Integer> empresasId) {
		Criteria query = getSession().createCriteria(getPersistentClass());
		query.createAlias("empresa", "emp");
		query.add(Restrictions.in("emp.empresaId", empresasId));
		query.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		
		return (List<FechamentoParamgeral>)query.list();
	}	

	@Override
	public List<FechamentoParamgeral> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

		return (List<FechamentoParamgeral>)c.list();
	}

	@Override
	public List<FechamentoParamgeral> buscaParametrosPorEmpresa(Integer empresaId) {
		Criteria query = getSession().createCriteria(getPersistentClass());
		query.createAlias("empresa", "emp");
		query.add(Restrictions.eq("emp.empresaId", empresaId));
		query.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
				
		return (List<FechamentoParamgeral>)query.list();
	}

	@Override
	public void borrar(FechamentoParamgeral entity) {
		entity.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entity.setFecmodif(Calendar.getInstance().getTime());
		entity.setActivo(Boolean.FALSE);
		actualizacion(entity);
	}
	
	@Override
	public FechamentoParamgeral buscaParametrosPorEmpresasBanco(Integer empresaId, String boletoBancoCod) {
		Criteria query = getSession().createCriteria(getPersistentClass());
		query.createAlias("empresa", "emp");
		query.add(Restrictions.eq("emp.empresaId", empresaId));
		query.add(Restrictions.eq("boletoBancoCod", boletoBancoCod));
		query.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		
		return (FechamentoParamgeral) query.uniqueResult();
	}
	
}
