package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@SequenceGenerator(name = "COMISSAO_RECEITA_SEQ", sequenceName = "COMISSAO_RECEITA_SEQ", allocationSize = 1)
@Table(name = "COMISSAO_RECEITA")
public class ComissaoReceita implements Serializable {
	
	private static final long serialVersionUID = 1L;

	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "COMISSAO_RECEITA_SEQ")
	@Column(name = "COMISSAORECEITA_ID")
	private Integer comissaoreceitaId;
	
	@Column(name = "DATAMOVIMENTO")
	@Temporal(TemporalType.DATE)
	private Date datamovimento;
	
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "PUNTOVENTA_ID")
	private PuntoVenta puntoVenta;
	
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "EMPRESA_ID")
	private Empresa empresa;
	
	@Column(name = "BPR")
	private BigDecimal bpr;
	
	@Column(name = "GAP")
	private BigDecimal gap;
	
	@Column(name = "IMPGAP")
	private BigDecimal impgap;
	
	@Column(name = "INTERNET")
	private BigDecimal internet;
	
	@Column(name = "CANCELADOS")
	private BigDecimal cancelados;
	
	@Column(name = "CANCELGAP")
	private BigDecimal cancelgap;
	
	@Column(name = "DEVOLVIDOS")
	private BigDecimal devolvidos;
	
	@Column(name = "DEVOLVIDOS_ORIGEM")
	private BigDecimal devolvidosOrigem;
	
	@Column(name = "DEVOLVIDOS_ORIGEM_GAP")
	private BigDecimal devolvidosOrigemGap;
	
	@Column(name = "DEVGAP")
	private BigDecimal devgap;
	
	@Column(name = "EXCESSO_BAGAGEM")
	private BigDecimal excessoBagagem;
	
	@Column(name = "SEG_OPCIONAL")
	private BigDecimal segOpcional;
	
	@Column(name = "SEG_OBRIGATORIO")
	private BigDecimal segObrigatorio;
	
	@Column(name = "RECEITA_OUTROS")
	private BigDecimal receitaOutros;
	
	@Column(name = "ACTIVO")
	private Boolean activo;
	
	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;
	
	@Column(name = "USUARIO_ID")
	private Integer usuarioId;

	@Column(name = "COMISSAO_BPR")
	private BigDecimal comissaoBpr;

	@Column(name = "COMISSAO_ENTREGA_PASSAGEM")
	private BigDecimal comissaoEntregaPassagem;
	
	@Column(name = "COMISSAO_ENTREGA_BAGAGEM")
	private BigDecimal comissaoExcessoBagagem;
	
	@Column(name = "COMISSAO_SEG_OPCIONAL")
	private BigDecimal comissaoSegOpcional;
	
	@Column(name = "COMISSAO_OUTROS")
	private BigDecimal comissaoOutros;
	
	@Column(name = "COMISSAO_SEG_OBRIGATORIO")
	private BigDecimal comissaoSegObrigatorio;

	@Column(name = "QTDE_IMPGAP")
	private Integer qtdeImpgap;
	
	public ComissaoReceita() {
		super();
	}

	public ComissaoReceita(com.rjconsultores.ventaboletos.vo.comissao.ComissaoReceita comissaoReceitaVO) {
		this();
		this.activo = Boolean.TRUE;
		this.fecmodif = new Date();
		this.datamovimento = comissaoReceitaVO.getData();
		this.bpr = comissaoReceitaVO.getReceitaBPR();
		this.gap = comissaoReceitaVO.getReceitaGAP();
		this.impgap = comissaoReceitaVO.getGapImpressa();
		this.qtdeImpgap = comissaoReceitaVO.getContImpressa();
		this.internet = comissaoReceitaVO.getInternet();
		this.cancelados = comissaoReceitaVO.getCancelados();
		this.cancelgap = comissaoReceitaVO.getCanceladosGAP();
		this.devolvidos = comissaoReceitaVO.getDevolvidos();
		this.devgap = comissaoReceitaVO.getDevolvidosGAP();
		this.excessoBagagem = comissaoReceitaVO.getReceitaExcessoBagagem();
		this.segObrigatorio = comissaoReceitaVO.getReceitaSeguro();
		this.segOpcional = comissaoReceitaVO.getReceitaSeguroOpcional();
		this.receitaOutros = comissaoReceitaVO.getReceitaSeguroOutros();
		this.devolvidosOrigem = comissaoReceitaVO.getDevolucoesOrigem();
		this.devolvidosOrigemGap = comissaoReceitaVO.getDevolucoesOrigemGAP();
		
		this.comissaoBpr = comissaoReceitaVO.getComissaoBPRDiaria();
		this.comissaoEntregaPassagem = comissaoReceitaVO.getComissaoEntregaPassagemDiaria();
		this.comissaoExcessoBagagem = comissaoReceitaVO.getComissaoExcessoBagagemDiaria();
		this.comissaoSegOpcional = comissaoReceitaVO.getComissaoSegOpcionalDiaria();
		this.comissaoOutros = comissaoReceitaVO.getComissaoOutrosDiaria();
		this.comissaoSegObrigatorio = comissaoReceitaVO.getComissaoSeguroObrigatorioDiaria();
	}

	public Integer getComissaoreceitaId() {
		return comissaoreceitaId;
	}

	public void setComissaoreceitaId(Integer comissaoreceitaId) {
		this.comissaoreceitaId = comissaoreceitaId;
	}

	public Date getDatamovimento() {
		return datamovimento;
	}

	public void setDatamovimento(Date datamovimento) {
		this.datamovimento = datamovimento;
	}

	public PuntoVenta getPuntoVenta() {
		return puntoVenta;
	}

	public void setPuntoVenta(PuntoVenta puntoVenta) {
		this.puntoVenta = puntoVenta;
	}

	public BigDecimal getBpr() {
		return bpr;
	}

	public void setBpr(BigDecimal bpr) {
		this.bpr = bpr;
	}

	public BigDecimal getGap() {
		return gap;
	}

	public void setGap(BigDecimal gap) {
		this.gap = gap;
	}

	public BigDecimal getImpgap() {
		return impgap;
	}

	public void setImpgap(BigDecimal impgap) {
		this.impgap = impgap;
	}

	public BigDecimal getInternet() {
		return internet;
	}

	public void setInternet(BigDecimal internet) {
		this.internet = internet;
	}

	public BigDecimal getCancelados() {
		return cancelados;
	}

	public void setCancelados(BigDecimal cancelados) {
		this.cancelados = cancelados;
	}

	public BigDecimal getCancelgap() {
		return cancelgap;
	}

	public void setCancelgap(BigDecimal cancelgap) {
		this.cancelgap = cancelgap;
	}

	public BigDecimal getDevolvidos() {
		return devolvidos;
	}

	public void setDevolvidos(BigDecimal devolvidos) {
		this.devolvidos = devolvidos;
	}

	public BigDecimal getDevgap() {
		return devgap;
	}

	public void setDevgap(BigDecimal devgap) {
		this.devgap = devgap;
	}

	public BigDecimal getExcessoBagagem() {
		return excessoBagagem;
	}

	public void setExcessoBagagem(BigDecimal excessoBagagem) {
		this.excessoBagagem = excessoBagagem;
	}

	public BigDecimal getSegOpcional() {
		return segOpcional;
	}

	public void setSegOpcional(BigDecimal segOpcional) {
		this.segOpcional = segOpcional;
	}

	public BigDecimal getReceitaOutros() {
		return receitaOutros;
	}

	public void setReceitaOutros(BigDecimal receitaOutros) {
		this.receitaOutros = receitaOutros;
	}

	public Boolean getActivo() {
		return activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Integer getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}

	public BigDecimal getDevolvidosOrigem() {
		return devolvidosOrigem;
	}

	public void setDevolvidosOrigem(BigDecimal devolvidosOrigem) {
		this.devolvidosOrigem = devolvidosOrigem;
	}

	public BigDecimal getDevolvidosOrigemGap() {
		return devolvidosOrigemGap;
	}

	public void setDevolvidosOrigemGap(BigDecimal devolvidosOrigemGap) {
		this.devolvidosOrigemGap = devolvidosOrigemGap;
	}

	public BigDecimal getComissaoBpr() {
		return comissaoBpr;
	}

	public void setComissaoBpr(BigDecimal comissaoBpr) {
		this.comissaoBpr = comissaoBpr;
	}

	public BigDecimal getComissaoEntregaPassagem() {
		return comissaoEntregaPassagem;
	}

	public void setComissaoEntregaPassagem(BigDecimal comissaoEntregaPassagem) {
		this.comissaoEntregaPassagem = comissaoEntregaPassagem;
	}

	public BigDecimal getComissaoExcessoBagagem() {
		return comissaoExcessoBagagem;
	}

	public void setComissaoExcessoBagagem(BigDecimal comissaoExcessoBagagem) {
		this.comissaoExcessoBagagem = comissaoExcessoBagagem;
	}

	public BigDecimal getComissaoSegOpcional() {
		return comissaoSegOpcional;
	}

	public void setComissaoSegOpcional(BigDecimal comissaoSegOpcional) {
		this.comissaoSegOpcional = comissaoSegOpcional;
	}

	public BigDecimal getComissaoOutros() {
		return comissaoOutros;
	}

	public void setComissaoOutros(BigDecimal comissaoOutros) {
		this.comissaoOutros = comissaoOutros;
	}

	public Integer getQtdeImpgap() {
		return qtdeImpgap;
	}

	public void setQtdeImpgap(Integer qtdeImpgap) {
		this.qtdeImpgap = qtdeImpgap;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((comissaoreceitaId == null) ? 0 : comissaoreceitaId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ComissaoReceita))
			return false;
		ComissaoReceita other = (ComissaoReceita) obj;
		if (comissaoreceitaId == null) {
			if (other.comissaoreceitaId != null)
				return false;
		} else if (!comissaoreceitaId.equals(other.comissaoreceitaId))
			return false;
		return true;
	}

	public Empresa getEmpresa() {
		return empresa;
	}

	public void setEmpresa(Empresa empresa) {
		this.empresa = empresa;
	}

	public BigDecimal getComissaoSegObrigatorio() {
		return comissaoSegObrigatorio;
	}

	public void setComissaoSegObrigatorio(BigDecimal comissaoSegObrigatorio) {
		this.comissaoSegObrigatorio = comissaoSegObrigatorio;
	}

	public BigDecimal getSegObrigatorio() {
		return segObrigatorio != null ? segObrigatorio : BigDecimal.ZERO ;
	}

	public void setSegObrigatorio(BigDecimal segObrigatorio) {
		this.segObrigatorio = segObrigatorio;
	}
	
}
