package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.AuditarEntidade;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@AuditarClasse(nome = "ConfRestricaoExcecao", tela = "auditarClasse.ConfRestricaoExcecao")
@SequenceGenerator(name = "CONF_RESTRICAO_EXCECAO_SEQ", sequenceName = "CONF_RESTRICAO_EXCECAO_SEQ", allocationSize = 1)
@Table(name = "CONF_RESTRICAO_EXCECAO")
public class ConfRestricaoExcecao implements Serializable, Auditavel<ConfRestricaoExcecao> {

	private static final long serialVersionUID = 1L;
	@Id
	@Basic(optional = false)
	@Column(name = "CONFRESTRICAOEXCECAO_ID")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "CONF_RESTRICAO_EXCECAO_SEQ")
	private Integer confRestricaoExcecaoId;
	@JoinColumn(name = "CONFRESTRICAOCANALVENTA_ID", referencedColumnName = "CONFRESTRICAOCANALVENTA_ID")
	@ManyToOne
	private ConfRestricaoCanalVenta confRestricaoCanalVenta;
	@Column(name = "FECEXCECAO")
	private Date fecExcecao;
	@Column(name = "ACTIVO")
	private Boolean activo;
	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;
	@Column(name = "USUARIO_ID")
	private Integer usuarioId;
	@NaoAuditar
	@JoinColumn(name = "CONFRESTRICAOTRAMO_ID", referencedColumnName = "CONFRESTRICAOTRAMO_ID")
	@OneToOne
	private ConfRestricaoTramo confRestricaoTramo;
	@NaoAuditar
	@Transient
	private ConfRestricaoExcecao confRestricaoExcecaoClone;
	

	public Integer getConfRestricaoExcecaoId() {
		return confRestricaoExcecaoId;
	}

	public void setConfRestricaoExcecaoId(Integer confRestricaoExcecaoId) {
		this.confRestricaoExcecaoId = confRestricaoExcecaoId;
	}

	public ConfRestricaoCanalVenta getConfRestricaoCanalVenta() {
		return confRestricaoCanalVenta;
	}

	public void setConfRestricaoCanalVenta(ConfRestricaoCanalVenta confRestricaoCanalVenta) {
		this.confRestricaoCanalVenta = confRestricaoCanalVenta;
	}

	public Date getFecExcecao() {
		return fecExcecao;
	}

	public void setFecExcecao(Date fecExcecao) {
		this.fecExcecao = fecExcecao;
	}

	public Boolean getActivo() {
		return activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Integer getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}

	public ConfRestricaoTramo getConfRestricaoTramo() {
		return confRestricaoTramo;
	}

	public void setConfRestricaoTramo(ConfRestricaoTramo confRestricaoTramo) {
		this.confRestricaoTramo = confRestricaoTramo;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activo == null) ? 0 : activo.hashCode());
		result = prime * result + ((confRestricaoTramo == null) ? 0 : confRestricaoTramo.hashCode());
		result = prime * result + ((fecExcecao == null) ? 0 : fecExcecao.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ConfRestricaoExcecao)) {
			return false;
		}
		ConfRestricaoExcecao other = (ConfRestricaoExcecao) obj;
		if (activo == null) {
			if (other.activo != null) {
				return false;
			}
		} else if (!activo.equals(other.activo)) {
			return false;
		}
		if (confRestricaoTramo == null) {
			if (other.confRestricaoTramo != null) {
				return false;
			}
		} else if (!confRestricaoTramo.equals(other.confRestricaoTramo)) {
			return false;
		}
		if (fecExcecao == null) {
			if (other.fecExcecao != null) {
				return false;
			}
		} else if (!fecExcecao.equals(other.fecExcecao)) {
			return false;
		}
		return true;
	}
	@Override
	public void clonar() throws CloneNotSupportedException {
		
		confRestricaoExcecaoClone = new ConfRestricaoExcecao();
		confRestricaoExcecaoClone = (ConfRestricaoExcecao) this.clone();
		Hibernate.initialize(confRestricaoExcecaoClone.getConfRestricaoCanalVenta());
		
	}

	@Override
	public ConfRestricaoExcecao getCloneObject() throws CloneNotSupportedException {
		return confRestricaoExcecaoClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getConfRestricaoTramo().getOrigem().getDescparada()+"-"+getConfRestricaoTramo().getDestino().getDescparada());
	}

}
