package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import com.rjconsultores.ventaboletos.enums.comissao.StatusLogConferencia;
import com.rjconsultores.ventaboletos.enums.comissao.TipoLogConferencia;

@Entity
@Table(name = "LOG_CONFERENCIA")
@SequenceGenerator(name = "LOG_CONFERENCIA_SEQ", sequenceName = "LOG_CONFERENCIA_SEQ", allocationSize = 1)
public class LogConferencia implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "LOG_CONFERENCIA_SEQ")
	@Column(name = "LOGCONFERENCIA_ID")
	private Long logconferenciaId;

	@Column(name = "OBSERVACAO")
	private String observacao;

	@Column(name = "PRECO")
	private BigDecimal preco;

	@Column(name = "ACTIVO")
	private boolean activo;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "FECMODIF")
	private Date fecmodif;

	@Column(name = "BOLETO_ID")
	private Long boletoId;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "USUARIO_ID")
	private Usuario usuario;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "CONFERENCIA_ID")
	private Conferencia conferencia;

	@Enumerated(EnumType.ORDINAL)
	@Column(name = "STATUS")
	private StatusLogConferencia status;
	
    @Column(name = "EVENTOEXTRA_ID")
    private Long eventoextraId; 

    @Column(name = "OCD_ID")
    private Long ocdId;

    @Enumerated(EnumType.ORDINAL)
    @Column(name = "TIPO")
    private TipoLogConferencia tipo;
    
    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "CONTACORRENTEPTOVTA_ID")
    private ContaCorrentePtoVta contaCorrentePtoVta;
    
    @Column(name = "INDCREDITO")
    private Short indcredito;
        
    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "TIPOINFORMATIVOCOMISSAO_ID")
    private TipoInformativoComissao tipoInformativoComissao;
    
    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "CONFERENCIAPENDENCIA_ID")
    private ConferenciaPendencia conferenciaPendencia;
    
    @Column(name = "INDMESMODIA")
    private Boolean indmesmodia;

	@Column(name = "CAJA_ID")
	private Long cajaId;
	
	@Column(name = "TRANSACAOORIGINAL_ID", precision = 15, scale = 0)
	private Long transacaoOriginalId;

	public Long getLogconferenciaId() {
		return logconferenciaId;
	}

	public void setLogconferenciaId(Long logconferenciaId) {
		this.logconferenciaId = logconferenciaId;
	}

	public String getObservacao() {
		return observacao;
	}

	public void setObservacao(String observacao) {
		this.observacao = observacao;
	}

	public BigDecimal getPreco() {
		return preco;
	}

	public void setPreco(BigDecimal preco) {
		this.preco = preco;
	}

	public boolean isActivo() {
		return activo;
	}

	public void setActivo(boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	public Conferencia getConferencia() {
		return conferencia;
	}

	public void setConferencia(Conferencia conferencia) {
		this.conferencia = conferencia;
	}

	public StatusLogConferencia getStatus() {
		return status;
	}

	public void setStatus(StatusLogConferencia status) {
		this.status = status;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((logconferenciaId == null) ? 0 : logconferenciaId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		LogConferencia other = (LogConferencia) obj;
		if (logconferenciaId == null) {
			if (other.logconferenciaId != null)
				return false;
		} else if (!logconferenciaId.equals(other.logconferenciaId))
			return false;
		return true;
	}

	public Long getEventoextraId() {
		return eventoextraId;
	}

	public void setEventoextraId(Long eventoextraId) {
		this.eventoextraId = eventoextraId;
	}

	public Long getOcdId() {
		return ocdId;
	}

	public void setOcdId(Long ocdId) {
		this.ocdId = ocdId;
	}

	public TipoLogConferencia getTipo() {
		return tipo;
	}

	public void setTipo(TipoLogConferencia tipo) {
		this.tipo = tipo;
	}

	public Long getBoletoId() {
		return boletoId;
	}

	public void setBoletoId(Long boletoId) {
		this.boletoId = boletoId;
	}

	public ContaCorrentePtoVta getContaCorrentePtoVta() {
		return contaCorrentePtoVta;
	}

	public void setContaCorrentePtoVta(ContaCorrentePtoVta contaCorrentePtoVta) {
		this.contaCorrentePtoVta = contaCorrentePtoVta;
	}

	/**
	 * <b>Tipos de movimentos</b><br>
	 * 0 - Débito<br>
	 * 1 - Crédito<br>
	 * 2 - Informativo<br>
	 */
	public Short isIndcredito() {
		return indcredito;
	}

	public void setIndcredito(Short indcredito) {
		this.indcredito = indcredito;
	}

	public TipoInformativoComissao getTipoInformativoComissao() {
		return tipoInformativoComissao;
	}

	public void setTipoInformativoComissao(TipoInformativoComissao tipoInformativoComissao) {
		this.tipoInformativoComissao = tipoInformativoComissao;
	}

	public ConferenciaPendencia getConferenciaPendencia() {
		return conferenciaPendencia;
	}

	public void setConferenciaPendencia(ConferenciaPendencia conferenciaPendencia) {
		this.conferenciaPendencia = conferenciaPendencia;
	}

	public Short getIndcredito() {
		return indcredito;
	}

	public Boolean getIndmesmodia() {
		return indmesmodia;
	}

	public void setIndmesmodia(Boolean indmesmodia) {
		this.indmesmodia = indmesmodia;
	}

	public Long getCajaId() {
		return cajaId;
	}

	public void setCajaId(Long cajaId) {
		this.cajaId = cajaId;
	}

	public Long getTransacaoOriginalId() {
		return transacaoOriginalId;
	}

	public void setTransacaoOriginalId(Long transacaoOriginalId) {
		this.transacaoOriginalId = transacaoOriginalId;
	}
	
}
