package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CasetaPeajeDAO;
import com.rjconsultores.ventaboletos.entidad.CasetaPeaje;
import com.rjconsultores.ventaboletos.entidad.CasetaPeajeExcepcion;
import com.rjconsultores.ventaboletos.service.CasetaPeajeService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("casetaPeajeService")
public class CasetaPeajeServiceImpl implements CasetaPeajeService {

	@Autowired
	private CasetaPeajeDAO casetaPeajeDAO;
	
	@Override
	public List<CasetaPeaje> obtenerTodos() {
		return casetaPeajeDAO.obtenerTodos();
	}
	
	@Override
	public List<CasetaPeaje> buscarTodosOrdenadoPorDescricao() {
		return casetaPeajeDAO.buscarTodosOrdenadoPorDescricao();
	}
	
	@Transactional
	public CasetaPeaje obtenerID(Integer id) {
		return casetaPeajeDAO.obtenerID(id);
	}

	@Transactional
	public CasetaPeaje suscribir(CasetaPeaje entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		return casetaPeajeDAO.suscribir(entidad);
	}

	@Transactional
	public CasetaPeaje actualizacion(CasetaPeaje entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		return casetaPeajeDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(CasetaPeaje entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);
		casetaPeajeDAO.actualizacion(entidad);
	}

	@Override
	public List<CasetaPeaje> buscar(String descricaoCasetaPeaje) {
		return casetaPeajeDAO.buscar(descricaoCasetaPeaje);
	}

	@Override
	public Boolean validarDescricaoCasetaPeaje(CasetaPeaje casetaPeaje) {
		List<CasetaPeaje> pracas = buscar(casetaPeaje.getDesccaseta());
		
		if(casetaPeaje.getCasetaPeajeId() == null) {
			if(pracas.isEmpty()){
				return Boolean.TRUE;
			}
		} else {
			if(pracas.isEmpty() || pracas.get(0).getCasetaPeajeId().equals(casetaPeaje.getCasetaPeajeId())) {
				return Boolean.TRUE;
			} else {
				return Boolean.FALSE;
			}
		}		
		
		return Boolean.FALSE;
	}

	@Override
	public void deletarVigencias(List<CasetaPeajeExcepcion> cpes) {
		casetaPeajeDAO.deletarVigencias(cpes);
	}

	@Override
	@Transactional
	public CasetaPeaje suscribirOrActualizacion(CasetaPeaje casetaPeaje) {
		if(casetaPeaje.getCasetaPeajeId() == null) {
			return suscribir(casetaPeaje);
		} else {
			return actualizacion(casetaPeaje);
		}
	}
}