package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CotacaoDAO;
import com.rjconsultores.ventaboletos.entidad.Cotacao;
import com.rjconsultores.ventaboletos.entidad.Moneda;
import com.rjconsultores.ventaboletos.service.CotacaoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("cotacaoService")
public class CotacaoServiceImpl implements CotacaoService {

	private static final Logger log = LogManager.getLogger(CotacaoServiceImpl.class);

	@Autowired
	private CotacaoDAO cotacaoDAO;

	public List<Cotacao> obtenerTodos() {
		return cotacaoDAO.obtenerTodos();
	}

	public Cotacao obtenerID(Integer id) {
		return cotacaoDAO.obtenerID(id);
	}

	@Transactional
	public Cotacao suscribir(Cotacao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return cotacaoDAO.suscribir(entidad);
	}

	@Transactional
	public Cotacao actualizacion(Cotacao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return cotacaoDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(Cotacao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		cotacaoDAO.actualizacion(entidad);
	}

	@Override
	public void inativarCotacoesAntigas(Moneda moneda) {

		cotacaoDAO.inativarCotacoesAntigas(moneda);

	}

}
