/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.MonedaDAO;
import com.rjconsultores.ventaboletos.entidad.Moneda;
import com.rjconsultores.ventaboletos.service.MonedaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

/**
 *
 * @author Rafius
 */
@Service("monedaService")
public class MonedaServiceImpl implements MonedaService {

	private static final Logger log = LogManager.getLogger(MonedaServiceImpl.class);
	
    @Autowired
    private MonedaDAO monedaDAO;

    public List<Moneda> obtenerTodos() {
        return monedaDAO.obtenerTodos();
    }

    public Moneda obtenerID(Integer id) {
        return monedaDAO.obtenerID(id);
    }

    @Transactional
    public Moneda suscribir(Moneda entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return monedaDAO.suscribir(entidad);
    }

    @Transactional
    public Moneda actualizacion(Moneda entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return monedaDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(Moneda entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.FALSE);

        monedaDAO.actualizacion(entidad);
    }

    public List<Moneda> buscar(String descmoneda) {
        return monedaDAO.buscar(descmoneda);
    }
    
	@Override
	public List<Moneda> obterTodosExcetoReais() {
		try {
			List<Moneda> monedas = monedaDAO.obtenerTodos();

			Moneda real = new Moneda();
			real.setMonedaId(1);

			monedas.remove(real);

			return monedas;
		} catch (Exception ex) {
			log.error(ex.getMessage(), ex);
			return new ArrayList<Moneda>();
		}
	}

	@Override
	public List<Moneda> obtenerTodosSortedByID() {
		try {
			List<Moneda> monedas = monedaDAO.obtenerTodos();
			Collections.sort(monedas, new Comparator<Moneda>() {
				@Override
				public int compare(Moneda o1, Moneda o2) {
					return o1.getMonedaId().compareTo(o2.getMonedaId());
				}
			});
			return monedas;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return new ArrayList<Moneda>(); 
	}

  
}
